# This file is part of the DEPS/graph-includes package
#
# (c) 2006 Yann Dirson <ydirson@altern.org>
# Distributed under version 2 of the GNU GPL.

package graphincludes::transform::compatgroup;

use warnings;
use strict;

use graphincludes::graph::grouped;
use Carp qw(croak);

sub apply {
  my %args = @_;

  # sanity checks
  if (!defined $args{graphs}
      or scalar @{$args{graphs}} != 1) {
    use Data::Dumper; print STDERR Dumper(keys %args);
    croak "graphincludes::transform::compatgroup applies to a single graph";
  }

  my $src = $args{graphs}[0];
  my $lvl = $args{level};
  my $prj = $args{labeller};
  my @prev = map { $prj->{TRANSGRAPH}->get_node_from_name($_)->{DATA}
		     or croak "no graph named '$_'" } @{$args{previous}};

  my $new = new graphincludes::graph::grouped;

  foreach my $node ($src->get_nodes) {
    my $newnodename = $prj->filelabel($node->{LABEL}, $lvl);
    next if !defined $newnodename;

    my ($newnode, $newsubnode) = _register_node ($new, $node, $newnodename, $lvl, $prj, \@prev);

    # find the immediate subedge
    foreach my $edge ($src->get_edges_from($node->{LABEL})) {
      my $newdepname = $prj->filelabel($edge->{DST}{LABEL}, $lvl);
      next if !defined $newdepname;

      my ($newdep, $newsubdep) = _register_node ($new, $edge->{DST}, $newdepname, $lvl, $prj, \@prev);

      # record the dep, unless it is intra-group
      unless ($newdep eq $newnode) {
	die "grumph for $node->{LABEL} -> $edge->{DST}{LABEL} ($newnodename -> $newdepname)"
	  unless $prev[$#prev]->has_edge($newsubnode->{LABEL}, $newsubdep->{LABEL});
	$new->record_edge($newnodename, $newdepname)->
	  add_ingredients($prev[$#prev]->get_edge($newsubnode->{LABEL}, $newsubdep->{LABEL}));
      }
    }
  }

  return $new;
}

# Hack to get the dependency graph in corect shape.  Not to be
# mis-used: the compatgroup mechanism allows for inconsistencies if
# this fixup is used, when the filelabel function does not define a
# proper group hierarchy.
sub fixup_dep {
  my ($graph, $src, $dst, $orig) = @_;

  $graph->has_edge($orig, $dst) or croak "no dependency exists from $orig to $dst";

  $graph->drop_edge($orig, $dst);
  $graph->record_edge($src, $dst);
}

sub _register_node {
  my ($new, $node, $newnodename, $lvl, $prj, $prev) = @_;

  # find the immediate subnode
  my ($subnode, $prevlevel);
  for ($prevlevel = $lvl - 1;
       $prevlevel >= 0 and
       !defined ($subnode = $prev->[$prevlevel]->
		 get_node_from_name($prj->filelabel($node->{LABEL}, $prevlevel)));
       --$prevlevel) {
  }

  # hack necessary because prefixstrip is applied to level 0 - we
  # would need another graph between 0 and 1 for this
  $subnode = $prev->[$prevlevel]->get_node_from_name($node->{LABEL}) unless defined $subnode;
  die "No ingredient node found for $newnodename for $node->{LABEL}" unless defined $subnode;

  my $newnode = $new->record_node($newnodename);
  $new->register_group_member($newnode, $subnode);
  return ($newnode->add_ingredients($subnode), $subnode);
}

1;
